`define NUM_CORE 2

module oc_usbhostslave_top(handsome_clk, handsome_din_top, handsome_dout_top, clk, rst);
	input handsome_clk;
	input handsome_din_top;
	output handsome_dout_top;
	input clk;
	input rst;

localparam input_width = 20;
localparam output_width = 23;
localparam reducer_log4 = 3;
localparam reducer_log4_top = 4;

wire [`NUM_CORE-1:0] writeEn;
wire [`NUM_CORE-1:0] strobe_i;
wire [`NUM_CORE-1:0] ack_o;
wire [`NUM_CORE-1:0] hostSOFSentIntOut;
wire [`NUM_CORE-1:0] hostConnEventIntOut;
wire [`NUM_CORE-1:0] hostResumeIntOut;
wire [`NUM_CORE-1:0] hostTransDoneIntOut;
wire [`NUM_CORE-1:0] slaveSOFRxedIntOut;
wire [`NUM_CORE-1:0] slaveResumeIntOut;
wire [`NUM_CORE-1:0] slaveTransDoneIntOut;
wire [`NUM_CORE-1:0] slaveNAKSentIntOut;
wire [`NUM_CORE-1:0] USBWireDataOutTick;
wire [`NUM_CORE-1:0] USBWireDataInTick;
wire [`NUM_CORE-1:0] USBWireCtrlOut;
wire [`NUM_CORE-1:0] USBFullSpeed;
wire [7:0] address_i [`NUM_CORE-1:0];
wire [7:0] data_i [`NUM_CORE-1:0];
wire [1:0] USBWireDataIn [`NUM_CORE-1:0];
wire [7:0] data_o [`NUM_CORE-1:0];
wire [1:0] USBWireDataOut [`NUM_CORE-1:0];
wire [`NUM_CORE-1:0] handsome_din;
wire [`NUM_CORE-1:0] handsome_dout;

handsome_lfsr #(`NUM_CORE) input_lfsr_top (
			.clock(handsome_clk),
			.input_bit(handsome_din_top),
			.output_vector(handsome_din));

generate
begin:G1
	genvar i;
	for (i = 0; i < `NUM_CORE; i = i+1)
	begin :stamp_inst

		handsome_lfsr #(input_width) input_lfsr (
			.clock(handsome_clk),
			.input_bit(handsome_din[i]),
			.output_vector({writeEn[i],strobe_i[i],address_i[i],data_i[i],USBWireDataIn[i]}));

		oc_usbhostslave oc_stamp(
			.clk(clk),
			.rst(rst),
			.writeEn(writeEn[i]),
			.strobe_i(strobe_i[i]),
			.address_i(address_i[i]),
			.data_i(data_i[i]),
			.USBWireDataIn(USBWireDataIn[i]),
			.ack_o(ack_o[i]),
			.hostSOFSentIntOut(hostSOFSentIntOut[i]),
			.hostConnEventIntOut(hostConnEventIntOut[i]),
			.hostResumeIntOut(hostResumeIntOut[i]),
			.hostTransDoneIntOut(hostTransDoneIntOut[i]),
			.slaveSOFRxedIntOut(slaveSOFRxedIntOut[i]),
			.slaveResumeIntOut(slaveResumeIntOut[i]),
			.slaveTransDoneIntOut(slaveTransDoneIntOut[i]),
			.slaveNAKSentIntOut(slaveNAKSentIntOut[i]),
			.USBWireDataOutTick(USBWireDataOutTick[i]),
			.USBWireDataInTick(USBWireDataInTick[i]),
			.USBWireCtrlOut(USBWireCtrlOut[i]),
			.USBFullSpeed(USBFullSpeed[i]),
			.data_o(data_o[i]),
			.USBWireDataOut(USBWireDataOut[i]));

		handsome_reducer #(reducer_log4) output_reducer (
			.clock(handsome_clk),
			.input_vector({ack_o[i],hostSOFSentIntOut[i],hostConnEventIntOut[i],hostResumeIntOut[i],hostTransDoneIntOut[i],slaveSOFRxedIntOut[i],slaveResumeIntOut[i],slaveTransDoneIntOut[i],slaveNAKSentIntOut[i],USBWireDataOutTick[i],USBWireDataInTick[i],USBWireCtrlOut[i],USBFullSpeed[i],data_o[i],USBWireDataOut[i]}),
			.output_bit(handsome_dout[i]));

	end

handsome_reducer #(reducer_log4_top) output_reducer_top (
	.clock(handsome_clk),
	.input_vector(handsome_dout[`NUM_CORE-1:0]),
	.output_bit(handsome_dout_top));

end
endgenerate
endmodule
